"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var core_1 = require("@angular/core");
var router_1 = require("@angular/router");
var _ = require("lodash");
var TreeComponent = (function () {
    function TreeComponent(resolver, router) {
        this.resolver = resolver;
        this.router = router;
        this.clickEvent = new core_1.EventEmitter();
        this.extendEvent = new core_1.EventEmitter();
        this.tempSearch = [];
    }
    Object.defineProperty(TreeComponent.prototype, "treeDatas", {
        get: function () {
            return this._treeDatas;
        },
        set: function (v) {
            this._treeDatas = v;
        },
        enumerable: true,
        configurable: true
    });
    TreeComponent.prototype.ngOnInit = function () {
        if (!this.keyId) {
            this.keyId = 'id';
        }
        if (!this.keyName) {
            this.keyName = 'title';
        }
        if (!this.keyChild) {
            this.keyChild = 'subNode';
        }
    };
    TreeComponent.prototype.clickNode = function (event) {
        this.selectTreeNode(event);
        this.clickEvent.emit(event);
    };
    TreeComponent.prototype.extendNode = function (event) {
        this.extendEvent.emit(event);
    };
    TreeComponent.prototype.checkNode = function (event) {
        var childs = event[this.keyChild];
        var parents = this.findParentsNodeById(event[this.keyId]);
        if (event.checked) {
            /**
                   * 复选框选中该节点
                   * 1/所有子节点选中
                   * 2/父节点判断
                   *  2.1/父节点为false
                   *    2.1.1/父节点下的所有兄弟节点全部checked  >   父节点为true(继续判断父)
                   */
            childs && this._checkChildrenNode(childs, true);
            this._checkParentNode(parents);
        }
        else {
            /**
                   * 复选框移除该选中状态
                   * 1/所有子节点移除选中状态
                   * 2/父节点判断
                   *  2.1/父节点为true
                   *    2.1.1/父节点改为false（继续判断父）
                   *  2.2/父节点为false
                   *    2.2.1/不做处理
                   */
            childs && this._checkChildrenNode(childs, false);
            this._unCheckParentNode(parents);
        }
    };
    TreeComponent.prototype._unCheckParentNode = function (parents) {
        if (parents.length > 1) {
            // 非根节点
            var parent_1 = parents[1];
            if (parent_1.checked) {
                parent_1.checked = false;
                this._unCheckParentNode(parents.splice(1, parents.length));
            }
        }
    };
    TreeComponent.prototype._checkParentNode = function (parents) {
        if (parents.length > 1) {
            // 非根节点
            var parent_2 = parents[1];
            var brothers = parent_2[this.keyChild];
            var flag = true;
            for (var _i = 0, brothers_1 = brothers; _i < brothers_1.length; _i++) {
                var bro = brothers_1[_i];
                if (!bro.checked) {
                    flag = false;
                    break;
                }
            }
            if (flag) {
                parents[1].checked = true;
                this._checkParentNode(parents.splice(1, parents.length));
            }
        }
    };
    TreeComponent.prototype._checkChildrenNode = function (children, checkType) {
        for (var _i = 0, children_1 = children; _i < children_1.length; _i++) {
            var child = children_1[_i];
            child.checked = checkType;
            if (child[this.keyChild]) {
                this._checkChildrenNode(child[this.keyChild], checkType);
            }
        }
    };
    TreeComponent.prototype._findNodeById = function (id, source) {
        for (var i = 0; i < source.length; i++) {
            var theI = source[i];
            if (theI[this.keyId] == id) {
                return theI;
            }
            if (theI[this.keyChild]) {
                var res = this._findNodeById(id, theI[this.keyChild]);
                if (res) {
                    return res;
                }
            }
        }
    };
    TreeComponent.prototype._findNodeByAllName = function (name, source) {
        for (var i = 0; i < source.length; i++) {
            var theI = source[i];
            if (theI[this.keyName] == name) {
                this.tempSearch.push(theI);
            }
            if (theI[this.keyChild]) {
                this._findNodeByAllName(name, theI[this.keyChild]);
            }
        }
    };
    TreeComponent.prototype._findNodeByName = function (name, source) {
        for (var i = 0; i < source.length; i++) {
            var theI = source[i];
            if (theI[this.keyName].toLowerCase().indexOf(name.toLowerCase()) != -1) {
                this.tempSearch.push(theI);
            }
            if (theI[this.keyChild]) {
                this._findNodeByName(name, theI[this.keyChild]);
            }
        }
    };
    TreeComponent.prototype._getCheckedNodes = function (result, source) {
        for (var _i = 0, source_1 = source; _i < source_1.length; _i++) {
            var item = source_1[_i];
            if (item.checked) {
                result.push(item);
            }
            if (item[this.keyChild]) {
                this._getCheckedNodes(result, item[this.keyChild]);
            }
        }
    };
    TreeComponent.prototype._findParentNodeById = function (id, source, parent) {
        for (var i = 0; i < source.length; i++) {
            var theI = source[i];
            if (theI[this.keyId] == id) {
                return parent;
            }
            if (theI[this.keyChild]) {
                var res = this._findParentNodeById(id, theI[this.keyChild], theI);
                if (res) {
                    return res;
                }
            }
        }
    };
    TreeComponent.prototype._findParentsNodeById = function (result, id, source) {
        for (var i = 0; i < source.length; i++) {
            var theI = source[i];
            if (theI[this.keyId] == id) {
                result.push(theI);
                return theI;
            }
            if (theI[this.keyChild]) {
                var res = this._findParentsNodeById(result, id, theI[this.keyChild]);
                if (res) {
                    result.push(theI);
                    return res;
                }
            }
        }
    };
    TreeComponent.prototype._findChildrensNodeByNode = function (result, source) {
        result.push(source);
        if (source[this.keyChild]) {
            for (var i = 0; i < source[this.keyChild].length; i++) {
                this._findChildrensNodeByNode(result, source[this.keyChild][i]);
            }
        }
    };
    // 匹配搜索
    // 匹配搜索
    TreeComponent.prototype._matchSearchNodesByKey = 
    // 匹配搜索
    function (result, val, key, source) {
        for (var i = 0; i < source.length; i++) {
            var theI = source[i];
            if (theI[key] == val) {
                result.push(source[i]);
            }
            if (theI[this.keyChild]) {
                this._matchSearchNodesByKey(result, val, key, theI[this.keyChild]);
            }
        }
    };
    // 模糊搜索
    // 模糊搜索
    TreeComponent.prototype._fuzzySearchNodesByKey = 
    // 模糊搜索
    function (result, val, key, source) {
        for (var i = 0; i < source.length; i++) {
            var theI = source[i];
            if (theI[key].toLowerCase().indexOf(val.toLowerCase()) != -1) {
                result.push(source[i]);
            }
            if (theI[this.keyChild]) {
                this._fuzzySearchNodesByKey(result, val, key, theI[this.keyChild]);
            }
        }
    };
    // 根据ID查询节点
    // 根据ID查询节点
    TreeComponent.prototype.findNodeById = 
    // 根据ID查询节点
    function (id) {
        this.tempSearch = [];
        return this._findNodeById(id, this.treeDatas);
    };
    // 根据名称查询全匹配节点
    // 根据名称查询全匹配节点
    TreeComponent.prototype.findNodeByName = 
    // 根据名称查询全匹配节点
    function (name) {
        this.tempSearch = [];
        this._findNodeByName(name, this.treeDatas);
        return this.tempSearch;
    };
    // 根据名称模糊查询节点
    // 根据名称模糊查询节点
    TreeComponent.prototype.findNodeByAllName = 
    // 根据名称模糊查询节点
    function (name) {
        this.tempSearch = [];
        this._findNodeByAllName(name, this.treeDatas);
        return this.tempSearch;
    };
    // 获取父节点
    // 获取父节点
    TreeComponent.prototype.findParentNodeById = 
    // 获取父节点
    function (id) {
        return this._findParentNodeById(id, this.treeDatas);
    };
    //获取父节点链
    //获取父节点链
    TreeComponent.prototype.findParentsNodeById = 
    //获取父节点链
    function (id) {
        var result = [];
        this._findParentsNodeById(result, id, this.treeDatas);
        return result;
    };
    TreeComponent.prototype.findChildrensNodeByNode = function (node) {
        var result = [];
        this._findChildrensNodeByNode(result, node);
        return result;
    };
    // 插入节点
    // 插入节点
    TreeComponent.prototype.insertNode = 
    // 插入节点
    function (child, parent) {
        var targetData;
        var parents;
        if (parent) {
            parents = this.findParentsNodeById(parent[this.keyId]);
            if (parents && parents.length > 0) {
                if (!parents[0][this.keyChild]) {
                    parents[0][this.keyChild] = [];
                }
                targetData = parents[0][this.keyChild];
                for (var i = 0; i < parents.length; i++) {
                    parents[i].show = true;
                }
            }
        }
        else {
            // 在根节点下插入
            targetData = this.treeDatas;
        }
        targetData.push(child);
        return child;
    };
    //更新节点数据
    //更新节点数据
    TreeComponent.prototype.updateNodeData = 
    //更新节点数据
    function (data, keys) {
        if (!keys) {
            return;
        }
        var targetNode = this.findNodeById(data[this.keyId]);
        for (var i = 0; i < keys.length; i++) {
            if (keys[i] == this.keyChild) {
                continue;
            }
            targetNode[keys[i]] = data[keys[i]];
        }
    };
    //删除节点
    //删除节点
    TreeComponent.prototype.deleteNodeById = 
    //删除节点
    function (id) {
        var parentNode = this.findParentNodeById(id);
        if (!parentNode) {
            parentNode = this.treeDatas;
        }
        else {
            parentNode = parentNode[this.keyChild];
        }
        for (var i = 0; i < parentNode.length; i++) {
            if (parentNode[i][this.keyId] == id) {
                parentNode.splice(i, 1);
                break;
            }
        }
    };
    // 匹配搜索
    // 匹配搜索
    TreeComponent.prototype.matchSearchNodesByKey = 
    // 匹配搜索
    function (val, key) {
        var result = [];
        this._matchSearchNodesByKey(result, val, key, this.treeDatas);
        return result;
    };
    // 模糊搜索
    // 模糊搜索
    TreeComponent.prototype.fuzzySearchNodesByKey = 
    // 模糊搜索
    function (val, key) {
        this.removeLighthightNodes(this.treeDatas);
        if (val == '') {
            return [];
        }
        var result = [];
        this._fuzzySearchNodesByKey(result, val, key, this.treeDatas);
        this.expandNodes(result);
        this.lighthightNodes(result);
        return result;
    };
    //展开节点
    //展开节点
    TreeComponent.prototype.expandNode = 
    //展开节点
    function (node) {
        var parents = this.findParentsNodeById(node[this.keyId]);
        if (parents && parents.length > 0) {
            if (!parents[0][this.keyChild]) {
                parents[0][this.keyChild] = [];
            }
            for (var i = 0; i < parents.length; i++) {
                parents[i].show = true;
            }
        }
    };
    // 展开节点s
    // 展开节点s
    TreeComponent.prototype.expandNodes = 
    // 展开节点s
    function (nodes) {
        for (var i = 0; i < nodes.length; i++) {
            this.expandNode(nodes[i]);
        }
    };
    // 高亮节点s
    // 高亮节点s
    TreeComponent.prototype.lighthightNodes = 
    // 高亮节点s
    function (nodes) {
        for (var i = 0; i < nodes.length; i++) {
            nodes[i].lighthight = true;
        }
    };
    // 移除高亮s
    // 移除高亮s
    TreeComponent.prototype.removeLighthightNodes = 
    // 移除高亮s
    function (nodes) {
        for (var i = 0; i < nodes.length; i++) {
            delete nodes[i].lighthight;
            if (nodes[i][this.keyChild]) {
                this.removeLighthightNodes(nodes[i][this.keyChild]);
            }
        }
    };
    // 选中节点
    // 选中节点
    TreeComponent.prototype.selectTreeNode = 
    // 选中节点
    function (node) {
        this.selectedNode && this.deSelectTreeNode(this.selectedNode);
        var item = this.findNodeById(node[this.keyId]);
        if (item) {
            item.selected = true;
            this.selectedNode = item;
        }
    };
    // 移除选中
    // 移除选中
    TreeComponent.prototype.deSelectTreeNode = 
    // 移除选中
    function (node) {
        var item = this.findNodeById(node[this.keyId]);
        if (item) {
            delete item.selected;
        }
    };
    // 获取选中节点
    // 获取选中节点
    TreeComponent.prototype.getCheckedNodes = 
    // 获取选中节点
    function () {
        var result = [];
        this._getCheckedNodes(result, this.treeDatas);
        return result;
    };
    TreeComponent.prototype.moveUpwardsNode = function (node) {
        var _this = this;
        var brothers;
        var parent = this.findParentNodeById(node[this.keyId]);
        if (!parent) {
            parent = this.treeDatas;
            brothers = parent;
        }
        else {
            brothers = parent[this.keyChild];
        }
        var theIndex = _.findIndex(brothers, function (o) {
            return o[_this.keyId] == node[_this.keyId];
        });
        if (theIndex == 0) {
            return;
        }
        brothers.splice(theIndex, 1);
        brothers.splice(theIndex - 1, 0, node);
    };
    TreeComponent.prototype.moveDownNode = function (node) {
        var _this = this;
        var brothers;
        var parent = this.findParentNodeById(node[this.keyId]);
        if (!parent) {
            parent = this.treeDatas;
            brothers = parent;
        }
        else {
            brothers = parent[this.keyChild];
        }
        var theIndex = _.findIndex(brothers, function (o) {
            return o[_this.keyId] == node[_this.keyId];
        });
        if (theIndex == (brothers.length - 1)) {
            return;
        }
        brothers.splice(theIndex, 1);
        brothers.splice(theIndex + 1, 0, node);
    };
    TreeComponent.decorators = [
        { type: core_1.Component, args: [{
                    selector: 'dc-tree',
                    template: "\n    <div class=\"display-flex width-100 flex-direction-column\">\n      <div *ngFor=\"let node of treeDatas;let i = index\">\n        <div class=\"node-container\">\n          <dc-node [nodeData]=\"node\" [root]=\"true\" [keyChild]=\"keyChild\" [keyId]=\"keyId\" [keyName]=\"keyName\" [options]=\"options\"\n                   (clickEvent)=\"clickNode($event)\" (checkEvent)=\"checkNode($event)\" (extendEvent)=\"extendNode($event)\"\n                   [myIndex]=\"i\"></dc-node>\n        </div>\n      </div>\n    </div>\n\n  ",
                    styles: [
                        'tree-node-container {overflow: hidden;position: relative;}'
                    ]
                },] },
    ];
    /** @nocollapse */
    TreeComponent.ctorParameters = function () { return [
        { type: core_1.ComponentFactoryResolver, },
        { type: router_1.Router, },
    ]; };
    TreeComponent.propDecorators = {
        "clickEvent": [{ type: core_1.Output },],
        "extendEvent": [{ type: core_1.Output },],
        "options": [{ type: core_1.Input },],
        "treeDatas": [{ type: core_1.Input },],
        "keyId": [{ type: core_1.Input },],
        "keyName": [{ type: core_1.Input },],
        "keyChild": [{ type: core_1.Input },],
    };
    return TreeComponent;
}());
exports.TreeComponent = TreeComponent;
//# sourceMappingURL=tree.component.js.map